#!usr/bin/env python
# -*- coding: utf8 -*-

import re
import pandas as pd
import unicodedata
from itertools import zip_longest

ACUTE_ACCENT = 769
CONSONANTS = list('#bcdfghjklmnpqrstvwxyzñĉp̂')
VOWELS = list('aeiouäạ̈ạẹịọụáéíóúä́ạ̈́ạ́ẹ́ị́ọ́ụ́') # Character Unicode NFC normalised
MULTIGRAPHS = ['ty', 'dy', 'qu', 'ĉh', 'ch', 'ts', 'tŝ', 'sh']


def rec_join(joiner, items):
    if type(items) == list:
        return joiner.join([rec_join(joiner, item) for item in items])
    else:
        return joiner.join(items)


def merge_multigraphs(word):
    word_list = list(word)
    word_merged = []
    idx = 0
    while idx < len(list(word_list)):
        if (word[idx:idx + 2] in MULTIGRAPHS or
            (word[idx:idx + 1] in VOWELS and word[idx + 1:idx + 2] in VOWELS)):
            word_merged.append(''.join(word_list[idx:idx + 2]))
            idx += 2
        elif word[idx:idx+1] == "'":
            word_merged[-1] += "'"
            idx += 1
        else:
            word_merged.append(word_list[idx:idx + 1])
            idx += 1

    return word_merged


def strip_accents(word):
    norm_word = unicodedata.normalize('NFD', word)
    removed_accent = [c for c in norm_word if ord(c) != ACUTE_ACCENT]
    return unicodedata.normalize('NFC', ''.join(removed_accent))


def syllable_split(word):
    # word = strip_accents(word)
    word_merged = merge_multigraphs(word)

    vowels = [s for s in word_merged if s[0] not in CONSONANTS+MULTIGRAPHS]
    syllables = []
    syllable = []

    for item in word_merged:
        if item in vowels:
            syllable.append(item)
            syllables.append(syllable)
            syllable = []
        else:
            syllable.append(item)

    if syllable:
        syllables.append(syllable)

    # Break double consonant clusters
    final_syl = []
    for i_syl, syl in enumerate(syllables):
        if len(syl) > 2 and final_syl:
            if syl[0][0] in CONSONANTS and syl[1][0] in CONSONANTS:
                new_syl = syl[1:]
                print(syl[0], new_syl)
                final_syl[-1].append(syl[0])
                final_syl.append(new_syl)
        else:
            final_syl.append(syl)

    # Merge last consonant if lonely letter
    if len(final_syl[-1]) == 1:
        final_syl[-2].append(final_syl[-1][0])
        final_syl = final_syl[:-1]

    syllables = [rec_join('', syllable) for syllable in final_syl]

    return syllables


def identify_diff_syllables(path_csv, dry_run):
    csv_data = pd.read_csv(path_csv)

    for idx, row in csv_data.iterrows():
        word1 = unicodedata.normalize('NFC','{}'.format(row['word1.raw']).strip())
        word2 = unicodedata.normalize('NFC','{}'.format(row['word2.raw']).strip())

        words = [word1, word2]
        words_syl = [syllable_split(w) for w in words]

        # Find the syllable that differs
        diff_syllables = [syl for syl in zip(*words_syl) if strip_accents(syl[0]) != strip_accents(syl[1])]
        diff = []

        # For those, annotate every glottal
        if word1 == 'nan' or word2 == 'nan':
            nan_word_syl = [(s1 if s1!='nan' else "",
                             s2 if s2!='nan' else "") for s1,s2 in list(zip_longest(*words_syl, fillvalue=''))]

            diff_syllables = [syl for syl in nan_word_syl
                              if syl[0].count("'") or syl[1].count("'") # Keep syl with '
                              or syl[0].count("i") or syl[1].count("i")]# Keep syl with i

        for (syl_1, syl_2) in diff_syllables:
            diff.append('{}/{}'.format(''.join(syl_1),''.join(syl_2)))
        csv_data.loc[idx, 'key_syllables'] = ', '.join(diff)

    # Dump
    if not dry_run:
        csv_data.to_csv(path_csv, index=False)
    else:
        print(csv_data)

def main(**kwargs):
    identify_diff_syllables(**kwargs)


def _parse_args(argv):
    import argparse

    parser = argparse.ArgumentParser(description='Identify minimal syllable in a CSV between word1.raw and word2.raw.')
    parser.add_argument('--path-csv', help='Path to CSV file.')
    parser.add_argument('--dry-run', required=False, action="store_true", default=False)
    args = parser.parse_args(argv)

    return vars(args)


if __name__ == '__main__':
    import sys
    import logging

    pgrm_name, argv = sys.argv[0], sys.argv[1:]
    args = _parse_args(argv)

    try:
        main(**args)
        sys.exit(0)
    except Exception as e:
        logging.exception(e)
        sys.exit(1)
