#!usr/bin/env python
# -*- coding: utf8 -*-
#
# -----------------------------------------------------------------------------
#   File: stats_bible.py (as part of project TSIMANE-GLOTTAL)
#   Created: 11/01/2023 10:02
#   Last Modified: 11/01/2023 10:02
# -----------------------------------------------------------------------------
#   Author: William N. Havard
#           Postdoctoral Researcher
#
#   Mail  : william.havard@ens.fr / william.havard@gmail.com
#  
#   Institution: ENS / Laboratoire de Sciences Cognitives et Psycholinguistique
#
# ------------------------------------------------------------------------------
#   Description: 
#       • 
# -----------------------------------------------------------------------------

import re
import unicodedata
from collections import Counter
from distutils.util import strtobool
from pprint import pprint
from string import punctuation as string_punctuation
from unicodedata import normalize
import matplotlib.pyplot as plt
import pandas as pd
from seaborn import heatmap

import numpy as np
from tqdm import tqdm

from utils import text_read, walk_dir, TokTyp, ulen, ngram_iterator, uiter

punctuation = string_punctuation.replace("'","")+"—…¿”“¡"

MULTIGRAPHS = ['ty', 'dy', 'qu', 'q̂u', 'c̣h', 'ĉh', 'ch', 'ts', 'tŝ', 'sh']

def merge_multigraphs(word_list):
    word_merged = []
    idx = 0
    while idx < len(word_list):
        if (''.join(word_list[idx:idx + 2]) in MULTIGRAPHS):
            word_merged.append(''.join(word_list[idx:idx + 2]))
            idx += 2
        else:
            word_merged.append(''.join(word_list[idx:idx + 1]))
            idx += 1
    return word_merged


def read_bible_files(bible_files):
    lines = []
    vocabulary = set()
    distr = dict()
    for file in bible_files:
        bible_file_data = text_read(file)
        for line in bible_file_data:
            _, line_data = line.split('\t') # Verse Number\tVerse
            line_data = normalize('NFC', line_data)
            line_data = line_data.translate(str.maketrans('', '', punctuation))
            line_data = line_data.lower()
            line_data = line_data.replace(' ', '#')
            line_data = line_data.replace("#̣", "")
            line_data = line_data.replace('#', '')
            lines.append((file, line_data))

            for char in merge_multigraphs(list(uiter(line_data))):
                if len(char) == 1 and any([char.isnumeric(), unicodedata.category(char) == 'Mn']):
                    continue
                if char == "#̣":
                    continue
                distr.setdefault(char, 1)
                distr[char] += 1
                distr
    return lines, set(distr.keys()), distr

# TODO merge multigraphs

def generate_matrix(counts, vocab, type_token, remove_zeros, target):
    vocab = list(vocab)
    count = np.zeros((len(vocab), len(vocab)), dtype='float32')
    for count_key, count_value in counts.items():
        before, after = count_key

        if before.isnumeric() or after.isnumeric():
            continue

        if before not in vocab or after not in vocab:
            continue

        before_idx, after_idx = vocab.index(before), vocab.index(after)

        count[before_idx, after_idx] = count_value/sum(counts.values())*100

    if remove_zeros:
        cols = np.argwhere(np.sum(count, axis=0) < 0.01).flatten()
        rows = np.argwhere(np.sum(count, axis=1) < 0.01).flatten()
        count = np.delete(np.delete(count, cols, axis=1), rows, axis=0)
        labels_cols = [v for iv, v in enumerate(vocab) if iv not in cols]
        labels_rows = [v for iv, v in enumerate(vocab) if iv not in rows]
    else:
        labels_cols = vocab
        labels_rows = vocab


    argsorted_cols = np.argsort(np.sum(count, axis=0)).flatten()[::-1]
    argsorted_rows = np.argsort(np.sum(count, axis=1)).flatten()

    sorted_cols = [v for i in argsorted_cols for iv, v in enumerate(labels_cols) if iv == i]
    sorted_rows = [v for i in argsorted_rows for iv, v in enumerate(labels_rows) if iv == i]

    count = count[:, argsorted_cols]
    count = count[argsorted_rows, :]

    ax = heatmap(count, cmap="jet", xticklabels=sorted_cols, yticklabels=sorted_rows)
    ax.set(xlabel="<{}> is followed by".format(','.join(target)),
           ylabel="<{}> is preceded by".format(','.join(target)))
    plt.yticks(rotation=0)
    plt.xticks(rotation=0)
    plt.title("<{}> distribution for {}".format(','.join(target), type_token.value))
    plt.show()


def get_context(line, target):
    contexts = []
    # problem with digraph + several letter in the same word
    for ngram_line in ngram_iterator(merge_multigraphs(list(uiter(line))), 3):
        before, *middle, after = ngram_line
        if ''.join(middle) == target:
            contexts.append((before, after))
    return contexts


def get_contexts(targets, data, type_token):
    matches = []
    types = set()
    char_freq = Counter()


    for i_line, (file, line) in enumerate(data):
        for word in line.split('#'):
            if not word: continue

            word = "{}".format(word)
            for char in merge_multigraphs(list(uiter(word))):
                if len(char) == 1 and unicodedata.category(char) == "Mn":
                    continue
                char_freq.update([char])

            if type_token == TokTyp.TOKEN:
                for target in targets:
                    ctx = get_context(word, target)
                    if ctx:
                        matches.extend(ctx)
            else:
                types.update([word])

    if type_token == TokTyp.TYPE:
        for item in list(types):
            for target in targets:
                ctx = get_context(item, target)
                if ctx:
                    matches.extend(ctx)

    return matches, char_freq


def dump_frequencies(matches, targets, vocabulary):

    counted_matches = Counter(matches)
    before_char, after_char = zip(*matches)
    before_char_counts = Counter(before_char)
    after_char_counts = Counter(after_char)

    before_records = {v: 0 for v in vocabulary}
    after_records  = {v: 0 for v in vocabulary}

    with open('out/context_{}_all.txt'.format('-'.join(targets)), 'w') as out_file:
        match_cnt = counted_matches
        for before, after in sorted(list(set(matches)), key=lambda tup: match_cnt[tup], reverse=True):
            out_file.write('{}_{} ({})\n'.format(before, after,
                                                 match_cnt[(before, after)] / sum(match_cnt.values()) * 100))

    with open('out/context_{}_left.txt'.format('-'.join(targets)), 'w') as out_file:
        match_cnt = before_char_counts
        for before in sorted(list(set(before_char)), key=lambda tup: match_cnt[tup], reverse=True):
            before_prc = match_cnt[before] / sum(match_cnt.values()) * 100
            before_records[before] = before_prc
            out_file.write('{}_ ({})\n'.format(before, before_prc))

    with open('out/context_{}_right.txt'.format('-'.join(targets)), 'w') as out_file:
        match_cnt = after_char_counts
        for after in sorted(list(set(after_char)), key=lambda tup: match_cnt[tup], reverse=True):
            after_prc = match_cnt[after] / sum(match_cnt.values()) * 100
            after_records[after] = after_prc
            out_file.write('_{} ({})\n'.format(after,after_prc))

    return before_records, after_records


def _stat_bible(path_data, type_token, remove_zeros, targets, show_diag, dump_ctx, min_occ):
    bible_files = walk_dir(path_data, ext='txt', return_full_path=True)
    # bible_files = bible_files[:5]
    bible_data, vocabulary, distr = read_bible_files(bible_files)

    for letter, count in distr.items():
        print('{}: {}'.format(letter, count/sum(distr.values())*100))

    if targets:
        matches, char_freq = get_contexts(targets, bible_data, type_token)
        if dump_ctx:
            dump_frequencies(matches, targets, vocabulary)

    else:
        col_names = []
        rows = {v: [] for v in sorted(list(vocabulary))}
        for voc_item in tqdm(sorted(list(vocabulary)), total=len(vocabulary)):
            matches, char_freq = get_contexts([voc_item], bible_data, type_token)
            if matches and len(matches) > min_occ:
                before_records, after_records = dump_frequencies(matches, [voc_item], vocabulary)

                for k,v in before_records.items():
                    if k not in rows: continue
                    rows[k].append(v)
                for k,v in after_records.items():
                    if k not in rows: continue
                    rows[k].append(v)
                col_names.extend(['{}_after'.format(voc_item), '{}_before'.format(voc_item)])
        print(rows)

        df = pd.DataFrame([[k] + v for k, v in rows.items()], columns=['letter']+col_names)
        print(df)
        df.to_csv('out/frequencies_without_sep.csv', index=False)

    if show_diag:
        counted_matches = Counter(matches)
        generate_matrix(counted_matches, vocabulary, type_token, remove_zeros, targets)



def main(**kwargs):
    _stat_bible(**kwargs)


def _parse_args(argv):
    import argparse

    parser = argparse.ArgumentParser(description='Co-occurence statistics from Bible data.')
    parser.add_argument('--path-data', required=True, help='Path to Bible text files.')
    parser.add_argument('--type-token', required=True, help='Type or Token',
                        choices=list(TokTyp),
                        type=lambda x: TokTyp.from_value(x.lower()))
    parser.add_argument('--remove-zeros', action='store', required=True,
                       choices=[True, False],
                       type=lambda x: bool(strtobool(x.lower())))
    parser.add_argument('--targets', nargs='+', required=False)
    parser.add_argument('--dump-ctx', required=False, default=False, action='store_true',
                        help='Dump context to a separate file.')
    parser.add_argument('--show-diag', required=False, default=False, action='store_true',
                        help="Show a visual representation of the distribution")
    parser.add_argument('--min-occ', required=False, type=int, action='store', default=10,
                        help="Mininum number of occurences")
    args = parser.parse_args(argv)

    return vars(args)


if __name__ == '__main__':
    import sys
    import logging

    pgrm_name, argv = sys.argv[0], sys.argv[1:]
    args = _parse_args(argv)

    try:
        main(**args)
        sys.exit(0)
    except Exception as e:
        logging.exception(e)
        sys.exit(1)
