#!usr/bin/env python
# -*- coding: utf8 -*-

import os
import re
import pandas as pd
from pprint import pprint
from pympi.Praat import TextGrid

from utils import text_read, text_dump, walk_dir, extract_source_sentence_file, SENT_ID_PATTERN

SENT_ID_PATTERN = re.compile('sentid-pair(\d+).word(\d+).(.*)')
TIER_RENAME = {
    'keysyll': 'key-word',
    'keysyll_seg': 'key-syll-segment',
    'sound': 'key-syll-segment-sound',
}


def update_textgrids(path):
    all_textgrids = walk_dir(path, ext='.TextGrid')

    for textgrid_file in all_textgrids:
        tg_path, tg_filename = os.path.split(textgrid_file)
        dir_name = tg_path.split(os.sep)[-1]

        # Get metadata (apply twice if respeak session)
        target_sentence_file = extract_source_sentence_file(dir_name)
        if 'respeak' in textgrid_file:
            target_sentence_file = extract_source_sentence_file(target_sentence_file)

        # Get sentid, wordid, senttype
        tg_raw_filename = tg_filename.replace('.TextGrid', '')
        _, sentid, *_ = tg_raw_filename.split('_')
        sentid_matched = re.search(SENT_ID_PATTERN, sentid)
        if not sentid_matched: continue
        pair_id, word_id, sent_type = sentid_matched.group(1), sentid_matched.group(2), sentid_matched.group(3)

        # Read summary sentences
        target_summary_all = pd.read_csv('./extra/sentences/summary/{}.csv'.format(target_sentence_file))

        row_idx = target_summary_all[target_summary_all['pair_index'] == 'pair{}'.format(pair_id)].index
        assert len(row_idx) == 1, ValueError("More that one pair found for that pairID!")  # should never happen
        row_idx = int(row_idx[0])

        # Get sentence, target word, and target syllable
        target_summary_sent = target_summary_all.loc[row_idx, 'word{}.{}'.format(word_id, sent_type)]
        target_summary_word = target_summary_all.loc[row_idx, 'word{}.{}'.format(word_id, 'raw')]
        target_summary_syl = target_summary_all.loc[row_idx, "key_syllables"]

        # Read TextGrid files
        textgrid = TextGrid(textgrid_file)

        # Rename tiers
        names = []
        for tier in textgrid.tiers:
            tier.name = TIER_RENAME.get(tier.name, tier.name)
            names.append(tier.name)

        for _, tier_name in TIER_RENAME.items():
            if tier_name not in names:
                textgrid.add_tier(tier_name)
                print("Added {} {}".format(tier_name, textgrid_file))

        # Read transcription interval
        trans_tier = textgrid.get_tier('transcription')
        tg_intervals = trans_tier.get_all_intervals()
        assert len(tg_intervals) == 1
        tg_start, tg_end, tg_trans = tg_intervals[0]

        # Get key syllables to annotate for that word
        words_key_syl = []
        for item in target_summary_syl.split(','):
            item = item.strip()
            for syl in [item.split('/')]:
                words_key_syl.append(syl[int(word_id)-1])

        # Add this information to the target word to annotate
        word_annotated = target_summary_word[:]
        for syl in words_key_syl:
            word_annotated=word_annotated.replace(syl, '[{}]'.format(syl))

        # Update transcription
        if tg_trans != target_summary_sent:
            textgrid.get_tier("transcription").intervals = list(map(list, textgrid.get_tier("transcription").intervals))
            for i_interval, (_, _, interval_value) in enumerate(textgrid.get_tier("transcription").intervals):
                if interval_value == tg_trans:
                    textgrid.get_tier("transcription").intervals[i_interval][-1] = target_summary_sent

        # Update key-word
        textgrid.get_tier("key-word").intervals = list(map(list, textgrid.get_tier("key-word").intervals))
        if len(textgrid.get_tier("key-word").intervals) > 1: continue
        for i_interval, (_, _, interval_value) in enumerate(textgrid.get_tier("key-word").intervals):
            textgrid.get_tier("key-word").intervals[i_interval][-1] = word_annotated

        ordered_tiers = [textgrid.get_tier('transcription'),
                         textgrid.get_tier('key-word'),
                         textgrid.get_tier('key-syll-segment'),
                         textgrid.get_tier('key-syll-segment-sound')]
        textgrid.tiers = list(reversed(ordered_tiers))
        textgrid.to_file(textgrid_file)


def main(**kwargs):
    update_textgrids(**kwargs)


def _parse_args(argv):
    import argparse

    parser = argparse.ArgumentParser(description='Pre-annotated TextGrid by filling tiers with summary CSVs. This '
                                                 'file will only work on the TextGrids in annotations/textgrid/LAAC-LSCP/AC')
    parser.add_argument('--path', required=True,
                        help='Path to the directory whose file will be annotated.')
    args = parser.parse_args(argv)

    return vars(args)


if __name__ == '__main__':
    import sys
    import logging

    pgrm_name, argv = sys.argv[0], sys.argv[1:]
    args = _parse_args(argv)

    try:
        main(**args)
        sys.exit(0)
    except Exception as e:
        logging.exception(e)
        sys.exit(1)