#!usr/bin/env python
# -*- coding: utf8 -*-

import os
import re
from datetime import datetime
import git

import pandas as pd
from enum import Enum, unique

SENT_ID_PATTERN = re.compile('sentid-pair(\d+).word(\d+).(.*)')

DATE_FORMAT = "%d/%m/%Y %H:%M:%S"
now = lambda: datetime.now().strftime(DATE_FORMAT)
now_raw = lambda: now().replace(' ', '-').replace('/', '-').replace(':', '-')


def get_commit_sha():
    repo = git.Repo(search_parent_directories=True)
    sha = repo.head.object.hexsha
    return sha[:12]


@unique
class CustomEnum(Enum):
    # Put Enum values here

    @classmethod
    def __from(cls, label, attr):
        for type_const in list(cls):
            if getattr(type_const, attr) == label:
                return type_const
        raise ValueError('Unknown type {}!'.format(label))

    @classmethod
    def from_value(cls, label):
        return cls.__from(label, 'value')

    @classmethod
    def from_name(cls, label):
        return cls.__from(label, 'name')

    @classmethod
    def values(cls):
        return (getattr(o, 'value') for o in list(cls))

    @classmethod
    def names(cls):
        return (getattr(o, 'name') for o in list(cls))

    @classmethod
    def keys(cls):
        return cls.names()

    def __eq__(self, other):
        if type(other) == str:
            return other == self.name or other == self.value
        return super().__eq__(other)



def extract_source_sentence_file(dir_name):
    # Extract the source sentence file name from dir name (batch_1, mono_syl_batch_1, etc.)

    _session_type, _session, _speaker, *_target_sentence_file, _date, _time = dir_name.split('_')
    target_sentence_file = "_".join(_target_sentence_file)
    return target_sentence_file


def get_tg_metadata(tg_filename):
    tg_raw_filename = tg_filename.replace('.TextGrid', '')
    _, sentid, *_ = tg_raw_filename.split('_')
    sentid_matched = re.search(SENT_ID_PATTERN, sentid)

    if not sentid_matched:
        return

    pair_id, word_id, sent_type = sentid_matched.group(1), sentid_matched.group(2), sentid_matched.group(3)
    return pair_id, word_id, sent_type


def get_summary_data(target_sentence_file, pair_id, word_id, sent_type):
    # Read summary sentences
    summary_data = pd.read_csv('./extra/sentences/summary/{}.csv'.format(target_sentence_file))

    row_idx = summary_data[summary_data['pair_index'] == 'pair{}'.format(pair_id)].index
    assert len(row_idx) == 1, ValueError("More that one pair found for that pairID!")  # should never happen
    row_idx = int(row_idx[0])

    # Get sentence, target word, and target syllable
    target_summary_sent = summary_data.loc[row_idx, 'word{}.{}'.format(word_id, sent_type)]
    target_summary_word = summary_data.loc[row_idx, 'word{}.{}'.format(word_id, 'raw')]
    target_summary_syl = summary_data.loc[row_idx, "key_syllables"]

    return target_summary_sent, target_summary_word, target_summary_syl


def find_all(src, tgt):
    find = src.find(tgt)
    found = []
    # loop util we find no match
    removed_char = 0
    while find != -1:
        found.append(find+removed_char)
        src = src[find+1:]
        removed_char += find+1
        find = src.find(tgt)
    return found


def walk_dir(path, ext='', return_full_path=True):
    files = []
    for p, d, f in os.walk(path):
        for ff in f:
            if ff.endswith(ext):
                path_suffix = p.replace(os.path.commonprefix([path, p]), '').lstrip(os.sep)
                if return_full_path:
                    files.append(os.path.join(path, path_suffix, ff))
                else:
                    files.append(os.path.join(path_suffix, ff))
    return sorted(files)


def text_read(txt_path):
    with open(txt_path, 'r') as in_file:
        lines = list(in_file.readlines())
    return lines


def text_dump(txt_path, payload):
    if type(payload) is list:
        payload = ''.join(payload)

    with open(txt_path, 'w') as out_file:
        out_file.write(payload)
